# from prettyqt.qt import QtMultimedia
# from prettyqt import multimedia

from __future__ import annotations


# QtMultimedia.QAbstractPlanarVideoBuffer.__bases__ = (multimedia.AbstractVideoBuffer,)

# class AbstractPlanarVideoBuffer(QtMultimedia.QAbstractPlanarVideoBuffer):
#     pass
