"""eventfilters module.

Contains custom EventFilter classes
"""

from .hovericoneventfilter import HoverIconEventFilter
from .animatedtooltipeventfilter import AnimatedToolTipEventFilter


__all__ = ["HoverIconEventFilter", "AnimatedToolTipEventFilter"]
