"""Module containing custom animation classes."""

from .bounceanimation import BounceAnimation
from .fadeinanimation import FadeInAnimation
from .slideanimation import SlideAnimation


__all__ = ["BounceAnimation", "SlideAnimation", "FadeInAnimation"]
