# from prettyqt.qt import QtCore

# from prettyqt import core


from __future__ import annotations


# QtCore.QRandomGenerator64.__bases__ = (core.RandomGenerator,)

# class RandomGenerator64(QtCore.QRandomGenerator64):
#     pass
